require 'action_controller'
require 'ostruct'
require 'has_scope'

module Rails
  def self.application
    @application ||= begin
      routes = ActionDispatch::Routing::RouteSet.new
      OpenStruct.new(routes: routes, env_config: {})
    end
  end
end

module ControllerExampleGroup
  def self.included(base)
    base.extend ClassMethods
    base.send(:include, ActionController::TestCase::Behavior)

    base.prepend_before do
      @routes = Rails.application.routes
      @controller = described_class.new
    end
  end

  module ClassMethods
    def setup(*methods)
      methods.each do |method|
        if method.to_s =~ /^setup_(fixtures|controller_request_and_response)$/
          prepend_before { send method }
        else
          before         { send method }
        end
      end
    end

    def teardown(*methods)
      methods.each { |method| after { send method } }
    end
  end
end

Rails.application.routes.draw do
  resources :users, only: [:index]
end

class UsersController < ActionController::Base
  include Rails.application.routes.url_helpers

  sorted
  filtered
  ranged
  searched

  def index
    @users = apply_scopes(User).all
    # p @users.to_sql
    render json: @users
  end
end
